import { __assign, __makeTemplateObject, __rest, __spreadArrays } from "tslib";
import { faFilter } from '@fortawesome/free-solid-svg-icons/faFilter';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import castArray from 'lodash/castArray';
import includes from 'lodash/includes';
import isEqual from 'lodash/isEqual';
import reject from 'lodash/reject';
import * as React from 'react';
import styled from 'styled-components';
import { Button } from '../../';
import { useBreakpoint } from '../../hooks/useBreakpoint';
import { randomString } from '../../utils';
import { Box } from '../Box';
import { Flex } from '../Flex';
import Search from '../Search';
import { FilterModal } from './FilterModal';
import * as SchemaSieve from './SchemaSieve';
import Summary from './Summary';
import ViewsMenu from './ViewsMenu';
var SearchWrapper = styled.div(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n\tflex-basis: 500px;\n"], ["\n\tflex-basis: 500px;\n"])));
var FilterWrapper = styled(Box)(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n\tposition: relative;\n"], ["\n\tposition: relative;\n"])));
var flattenViews = function (views) {
    return views.map(function (_a) {
        var filters = _a.filters, view = __rest(_a, ["filters"]);
        return (__assign(__assign({}, view), { filters: filters.map(function (filter) { return SchemaSieve.flattenSchema(filter); }) }));
    });
};
var shouldRenderComponent = function (mode, renderMode) {
    if (!renderMode) {
        return true;
    }
    var allowedModes = castArray(renderMode);
    if (includes(allowedModes, 'all')) {
        return true;
    }
    return includes(allowedModes, mode);
};
var Filters = function (_a) {
    var filters = _a.filters, views = _a.views, schema = _a.schema, disabled = _a.disabled, viewScopes = _a.viewScopes, onFiltersUpdate = _a.onFiltersUpdate, onViewsUpdate = _a.onViewsUpdate, addFilterButtonProps = _a.addFilterButtonProps, viewsMenuButtonProps = _a.viewsMenuButtonProps, renderMode = _a.renderMode, dark = _a.dark, compact = _a.compact;
    var flatSchema = SchemaSieve.flattenSchema(schema);
    var flatViews = flattenViews(views || []);
    var flatFilters = (filters || []).map(function (filter) {
        return SchemaSieve.flattenSchema(filter);
    });
    var shouldHideLabel = useBreakpoint(compact || [false]);
    var _b = React.useState(false), showModal = _b[0], setshowModal = _b[1];
    var _c = React.useState(''), searchString = _c[0], setSearchString = _c[1];
    var _d = React.useState(), editingFilter = _d[0], setEditingFilter = _d[1];
    var _e = React.useState(flatFilters), filtersState = _e[0], setFiltersState = _e[1];
    var _f = React.useState(flatViews), viewsState = _f[0], setViewsState = _f[1];
    var _g = React.useState(flatSchema), schemaState = _g[0], setschemaState = _g[1];
    var _h = React.useState([
        SchemaSieve.getCleanEditModel(flatSchema),
    ]), edit = _h[0], setEdit = _h[1];
    React.useEffect(function () {
        return onFiltersUpdate &&
            onFiltersUpdate(filtersState.map(function (filter) { return SchemaSieve.unflattenSchema(filter); }));
    }, [filtersState]);
    React.useEffect(function () { return onViewsUpdate && onViewsUpdate(viewsState); }, [
        viewsState,
    ]);
    React.useEffect(function () {
        if (!isEqual(filtersState, flatFilters)) {
            setFiltersState(flatFilters);
        }
    }, [filters]);
    React.useEffect(function () {
        if (!isEqual(viewsState, flatViews)) {
            setViewsState(flatViews);
        }
    }, [views]);
    React.useEffect(function () {
        if (!isEqual(schemaState, flatSchema)) {
            setschemaState(flatSchema);
        }
    }, [schema]);
    var addFilter = function (edit) {
        var newFilter = SchemaSieve.createFilter(schema, edit);
        var currentFilters = !!editingFilter
            ? filtersState.map(function (filter) {
                return filter.$id === editingFilter ? newFilter : filter;
            })
            : __spreadArrays(filtersState, [newFilter]);
        setFiltersState(currentFilters);
        setEdit([SchemaSieve.getCleanEditModel(schemaState)]);
        setshowModal(false);
        setEditingFilter(null);
    };
    var editFilter = function (filter) {
        var signatures = SchemaSieve.decodeFilter(schemaState, filter);
        setEdit(signatures);
        setEditingFilter(filter.$id);
        setshowModal(true);
    };
    var removeFilter = function (_a) {
        var $id = _a.$id, title = _a.title;
        if (title === SchemaSieve.FULL_TEXT_SLUG) {
            setSearchString('');
        }
        setFiltersState(reject(filtersState, { $id: $id }));
    };
    var clearAllFilters = function () {
        setFiltersState([]);
        setSearchString('');
    };
    var saveView = function (name, scope) {
        var view = {
            id: randomString(),
            name: name,
            scope: scope,
            filters: filtersState,
        };
        setViewsState(function (prevViews) { return __spreadArrays(prevViews, [view]); });
    };
    var deleteView = function (_a) {
        var id = _a.id;
        setViewsState(reject(viewsState, { id: id }));
    };
    var setSimpleSearch = function (term) {
        var newFilters = term
            ? SchemaSieve.upsertFullTextSearch(schemaState, filtersState, term)
            : SchemaSieve.removeFullTextSearch(filtersState);
        setSearchString(term);
        setFiltersState(newFilters);
    };
    return (React.createElement(FilterWrapper, { mb: 3 },
        React.createElement(Flex, { justifyContent: "space-between" },
            shouldRenderComponent('add', renderMode) && (React.createElement(Button, __assign({ mr: 30, disabled: disabled, primary: true, icon: React.createElement(FontAwesomeIcon, { icon: faFilter }), onClick: function () {
                    setshowModal(true);
                    setEditingFilter(null);
                } }, addFilterButtonProps, { label: shouldHideLabel ? undefined : 'Add filter' }))),
            shouldRenderComponent('search', renderMode) && (React.createElement(SearchWrapper, null,
                React.createElement(Search, { dark: dark, disabled: disabled, value: searchString, onChange: function (e) {
                        return setSimpleSearch(e.target.value);
                    } }))),
            shouldRenderComponent('views', renderMode) && (React.createElement(ViewsMenu, { dark: dark, buttonProps: viewsMenuButtonProps, disabled: disabled, views: viewsState || flatViews, schema: schema, hasMultipleScopes: viewScopes && viewScopes.length > 1, setFilters: function (filters) { return setFiltersState(filters); }, deleteView: function (view) { return deleteView(view); }, renderMode: renderMode, shouldHideLabel: shouldHideLabel }))),
        showModal && (React.createElement(FilterModal, { addFilter: function (edit) { return addFilter(edit); }, onClose: function () { return setshowModal(false); }, schema: schemaState, edit: edit })),
        shouldRenderComponent('summary', renderMode) &&
            !!filtersState.length &&
            !disabled && (React.createElement(Summary, { scopes: viewScopes, edit: function (filter) { return editFilter(filter); }, delete: function (filter) { return removeFilter(filter); }, saveView: function (name, scope) { return saveView(name, scope); }, clearAllFilters: clearAllFilters, filters: filtersState, views: viewsState, schema: schemaState }))));
};
export default Filters;
var templateObject_1, templateObject_2;
